# File for module specific CMake tests.
#
# .. cmake_function:: add_generated_executable
#
#    .. cmake_param:: UFLFILE
#       :single:
#       :required:
#
#       The UFL file to create the executable from.
#
#    .. cmake_param:: TARGET
#       :single:
#       :required:
#
#       The name given to the added executable target.
#
#    .. cmake_param:: OPERATOR
#       :single:
#
#       The local operator file name to generate. Defaults
#       to a suitably mangled, but not easily readable name.
#
#    .. cmake_param:: SOURCE
#
#       The cc source file to build from. If omitted, a minimal
#       source file and a driver file will be generated.
#
#    .. cmake_param:: FORM_COMPILER_ARGS
#       :multi:
#       :argname arg:
#
#       Additional arguments as recognized by the form compiler.
#
#    .. cmake_param:: DEPENDS
#       :multi:
#       :argname dep:
#
#       Additional dependencies of the generated executable (changes in those
#       will retrigger generation)
#
#    Add an executable to the project that gets automatically
#    generated at configure time with the form compiler uf2pdelab.
#    Regeneration is triggered correctly if the UFL file or the
#    form compiler changed.
#

add_custom_target(generation)

# Gather a list of form compiler sources to add as dependencies
# to have correct retriggers of generated executables
if(CMAKE_PROJECT_NAME STREQUAL dune-perftool)
  set(UFL2PDELAB_GLOB_PATTERN "${CMAKE_SOURCE_DIR}/python/*.py")
else()
  dune_module_path(MODULE dune-perftool
                   RESULT perftool_path
                   CMAKE_MODULES)
  dune_execute_process(COMMAND ${CMAKE_BINARY_DIR}/run-in-dune-env python ${perftool_path}/perftool_sourcepath.py
                       OUTPUT_VARIABLE perftool_source
                       OUTPUT_STRIP_TRAILING_WHITESPACE
                       )
  set(UFL2PDELAB_GLOB_PATTERN "${perftool_source}/*.py")
endif()
file(GLOB_RECURSE UFL2PDELAB_SOURCES ${UFL2PDELAB_GLOB_PATTERN})

function(add_generated_executable)
  set(OPTIONS)
  set(SINGLE TARGET OPERATOR SOURCE UFLFILE)
  set(MULTI FORM_COMPILER_ARGS DEPENDS)
  include(CMakeParseArguments)
  cmake_parse_arguments(GEN "${OPTIONS}" "${SINGLE}" "${MULTI}" ${ARGN})

  if(GEN_UNPARSED_ARGUMENTS)
    message(FATAL_ERROR "Unrecognized arguments in add_generated_executable. This usually indicates a typo.")
  endif()

  # Apply defaults and enforce requirements
  if(NOT GEN_TARGET)
    message(FATAL_ERROR "Need to specify the TARGET parameter for add_generated_executable")
  endif()
  if(NOT GEN_UFLFILE)
    message(FATAL_ERROR "Need to specify the UFLFILE parameter for add_generated_executable")
  endif()
  if(NOT IS_ABSOLUTE GEN_UFLFILE)
    set(GEN_UFLFILE ${CMAKE_CURRENT_SOURCE_DIR}/${GEN_UFLFILE})
  endif()
  if(NOT GEN_OPERATOR)
    set(GEN_OPERATOR ${GEN_TARGET}_operator.hh)
    set(GEN_OPERATOR ${CMAKE_CURRENT_BINARY_DIR}/${GEN_OPERATOR})
  endif()
  if(NOT GEN_SOURCE)
    set(GEN_DRIVER ${GEN_TARGET}_driver.hh)
    set(GEN_FORM_COMPILER_ARGS ${GEN_FORM_COMPILER_ARGS} --driver-file ${GEN_DRIVER})
    set(GEN_SOURCE ${GEN_TARGET}_main.cc)
    dune_module_path(MODULE dune-perftool
                     RESULT perftool_path
                     CMAKE_MODULES)
    configure_file(${perftool_path}/StandardMain.cmake ${GEN_SOURCE})
  endif()

  add_custom_command(OUTPUT ${GEN_OPERATOR} ${GEN_DRIVER}
                     COMMAND ${CMAKE_BINARY_DIR}/run-in-dune-env ufl2pdelab
                             --project-basedir ${CMAKE_BINARY_DIR}
                             --operator-file ${GEN_OPERATOR}
                             ${GEN_FORM_COMPILER_ARGS}
                             --uflfile ${GEN_UFLFILE}
                     DEPENDS ${GEN_UFLFILE} ${UFL2PDELAB_SOURCES} ${GEN_DEPENDS} ${DUNE_PERFTOOL_ADDITIONAL_PYTHON_SOURCES}
                     COMMENT "Running ufl2pdelab for the target ${GEN_TARGET}"
                    )

  add_executable(${GEN_TARGET} ${GEN_SOURCE} ${GEN_OPERATOR})
  target_include_directories(${GEN_TARGET} PUBLIC ${CMAKE_CURRENT_BINARY_DIR})
  add_dependencies(generation ${GEN_TARGET})
endfunction()

include(GeneratedSystemtests)
